from typing import Generator
from functools import partial
from psycopg import Connection
import polars as pl
from typing import Any
from bollhav.modes import WriteMode
from bollhav.postgres.modes import (
    truncate_insert,
    overwrite_insert,
    update_insert,
    create_view,
    append,
)
from datetime import datetime


def write(
    conn: Connection,
    df_gen: Generator[pl.DataFrame, None, None],
    model: Any,
    since: datetime | None = None,
    until: datetime | None = None,
) -> None:
    match model.write_mode:
        case WriteMode.VIEW:
            write_function = create_view
        case WriteMode.APPEND:
            write_function = append
        case WriteMode.TRUNCATE_INSERT:
            write_function = truncate_insert
        case WriteMode.OVERWRITE_INSERT:
            write_function = partial(overwrite_insert, since=since, until=until)
        case WriteMode.UPDATE_INSERT:
            write_function = partial(update_insert)
        case _:
            raise ValueError(f"Unhandled write mode: {model.write_mode}")

    for df in df_gen:
        if len(df) == 0:
            continue
        write_function(conn=conn, model=model, df=df)



from unittest.mock import MagicMock, patch
import polars as pl
import pytest
from bollhav.postgres.write_modes import write


def test_invalid_write_mode():
    conn = MagicMock()
    model = MagicMock()
    model.write_mode = MagicMock()

    with pytest.raises(ValueError):
        write(conn=conn, df_gen=iter([pl.DataFrame({"a": [1]})]), model=model)



from bollhav.model import Model
from bollhav.modes import ModelType, WriteMode
from bollhav.batching import BatchSize
from bollhav.database import Database, DatabaseColumn

__all__ = ["Model", "ModelType", "WriteMode", "BatchSize", "Database", "DatabaseColumn"]
